"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BJSON = exports.stringify = exports.BJSONConstructor = exports.Serializer = exports.BJSON_FUNCTION = exports.BJSON_UNDEFINED = exports.SEP = void 0;
var dateSerialize_1 = require("./dateSerialize");
var getTypeName_1 = require("./getTypeName");
exports.SEP = 'ː';
function withSEP(txt) {
    return "".concat(exports.SEP).concat(txt).concat(exports.SEP);
}
exports.BJSON_UNDEFINED = withSEP('undef');
exports.BJSON_FUNCTION = withSEP('func');
var Serializer = /** @class */ (function () {
    function Serializer(formatter) {
        this.formatter = formatter;
        this.name = formatter.name;
        this.sep = Serializer.SEP;
        this.paramSep = this.sep;
        this.blockStart = "".concat(this.sep).concat(this.name).concat(this.sep, "(");
        this.blockEnd = ")";
    }
    Serializer.prototype.stringify = function (value) {
        var _this = this;
        var tested = this.formatter.match(value);
        if (tested === undefined)
            return;
        var wrap = function (txt) { return "".concat(_this.blockStart).concat(txt).concat(_this.blockEnd); };
        if (this.formatter.jsonReplacer) {
            return this.formatter.jsonReplacer;
        }
        var params = this.formatter.getParams(tested);
        return wrap(params.join(this.paramSep));
    };
    Serializer.prototype.parse = function (value) {
        var _a;
        var _b;
        var str = (_b = value === null || value === void 0 ? void 0 : value.toString) === null || _b === void 0 ? void 0 : _b.call(value);
        if (typeof str !== 'string')
            return;
        if (this.formatter.jsonReplacer !== undefined &&
            value === this.formatter.jsonReplacer) {
            return {
                value: this.formatter.hydrate(),
            };
        }
        if (!(str.startsWith(this.blockStart) && str.endsWith(this.blockEnd))) {
            return;
        }
        var params = value
            .slice(this.blockStart.length)
            .slice(0, -this.blockEnd.length)
            .split(this.paramSep);
        return {
            value: (_a = this.formatter).hydrate.apply(_a, __spreadArray([], __read(params), false)),
        };
    };
    Serializer.SEP = exports.SEP;
    return Serializer;
}());
exports.Serializer = Serializer;
var BJSONConstructor = /** @class */ (function () {
    function BJSONConstructor(options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this.serializers = [];
        this.stringify = function (value, options) {
            if (options === void 0) { options = {}; }
            var handler = options.handler, quoteValues = options.quoteValues, quoteKeys = options.quoteKeys;
            var self = _this;
            var str = stringify(value, __assign(__assign({}, options), { quoteValues: quoteValues, quoteKeys: quoteKeys, defaultHandler: function (_a) {
                    var value = _a.value, options = _a.options;
                    var serializer = _this.serializers.find(function (el) { return el.formatter.match(value) !== undefined; });
                    var handled = handler === null || handler === void 0 ? void 0 : handler(__assign(__assign({}, options), { value: value, self: self, serializer: serializer }));
                    if (handled !== undefined)
                        return handled;
                    if (!serializer)
                        return undefined;
                    return serializer.stringify(value);
                } }));
            return "".concat(str);
        };
        this.parse = function (input) {
            return JSON.parse(input, function (_key, value) {
                var e_1, _a;
                try {
                    for (var _b = __values(_this.serializers), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var serializer = _c.value;
                        var match = serializer.parse(value);
                        if (match !== undefined)
                            return match.value;
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                return value;
            });
        };
        this.serializers = __spreadArray(__spreadArray([], __read(BJSONConstructor.serializers), false), __read((options.extraSerializers || [])), false);
    }
    BJSONConstructor.prototype.getSerializer = function (value) {
        var _a, _b;
        var name = (_b = (_a = value === null || value === void 0 ? void 0 : value.split) === null || _a === void 0 ? void 0 : _a.call(value, "".concat(Serializer.SEP))) === null || _b === void 0 ? void 0 : _b[0];
        if (!name)
            return;
        return this.serializers.find(function (s) { return s.name === name; });
    };
    BJSONConstructor.prototype.tsName = function (value) {
        var _a;
        return (_a = this.getSerializer(value)) === null || _a === void 0 ? void 0 : _a.formatter.tsName();
    };
    BJSONConstructor.serializers = [
        new Serializer({
            name: 'Date',
            tsName: function () {
                return 'Date';
            },
            hydrate: function (value) {
                return new Date(value);
            },
            getParams: function (value) {
                return [value.toJSON()];
            },
            match: function (value) {
                return value instanceof Date
                    ? value
                    : typeof value === 'string'
                        ? (0, dateSerialize_1.dateSerialize)(value) || undefined
                        : undefined;
            },
        }),
        new Serializer({
            name: 'RegExp',
            tsName: function () {
                return 'RegExp';
            },
            hydrate: function (source, flags) {
                return new RegExp(source, flags);
            },
            getParams: function (value) {
                return [value.source, value.flags];
            },
            match: function (value) {
                return value instanceof RegExp ? value : undefined;
            },
        }),
        new Serializer({
            name: 'NaN',
            tsName: function () {
                return 'NaN';
            },
            hydrate: function () {
                return NaN;
            },
            getParams: function () {
                return [];
            },
            match: function (value) {
                return typeof value === 'number' && isNaN(value) ? value : undefined;
            },
        }),
        new Serializer({
            name: 'Undefined',
            tsName: function () {
                return 'undefined';
            },
            hydrate: function () {
                return undefined;
            },
            getParams: function () {
                return [];
            },
            match: function (value) {
                return value === undefined ? exports.BJSON_UNDEFINED : undefined;
            },
            jsonReplacer: exports.BJSON_UNDEFINED,
        }),
        new Serializer({
            name: 'Function',
            tsName: function () {
                return 'Function';
            },
            hydrate: function () {
                return function __unknown__() { };
            },
            getParams: function () {
                return [];
            },
            match: function (value) {
                return typeof value === 'function' ? value : undefined;
            },
            jsonReplacer: exports.BJSON_FUNCTION,
        }),
    ];
    return BJSONConstructor;
}());
exports.BJSONConstructor = BJSONConstructor;
// some parts from meteor ejson
function stringify(value, options) {
    var _a;
    if (options === void 0) { options = {}; }
    var defaultHandler = options.defaultHandler, _b = options.quoteValues, quoteValues = _b === void 0 ? function (v) { return JSON.stringify(v); } : _b, _c = options.quoteKeys, quoteKeys = _c === void 0 ? JSON.stringify : _c;
    var typeName = (0, getTypeName_1.getTypeName)(value);
    var info = { key: (_a = options === null || options === void 0 ? void 0 : options.key) !== null && _a !== void 0 ? _a : undefined };
    var handled = defaultHandler === null || defaultHandler === void 0 ? void 0 : defaultHandler({ value: value, options: options });
    if (handled !== undefined)
        return quoteValues(handled, info);
    switch (typeName) {
        case 'String':
            return quoteValues(value, info);
        case 'NaN':
        case 'Infinity':
            return 'null';
        case 'Number':
            return quoteValues(value, info);
        case 'Boolean':
            return quoteValues(value, info);
        case 'Array': {
            var partial = [];
            var v = // Is the value an array?
             void 0; // Is the value an array?
            // The value is an array. Stringify every element. Use null as a
            // placeholder for non-JSON values.
            var length_1 = value.length;
            for (var i = 0; i < length_1; i += 1) {
                partial[i] = stringify(value[i], __assign(__assign({}, options), { key: i })) || 'null';
            } // Join all the elements together, separated with commas, and wrap
            // them in brackets.
            if (partial.length === 0) {
                v = '[]';
            }
            else {
                v = '[' + partial.join(',') + ']';
            }
            return v;
        }
        case 'Null':
            return 'null';
        case 'Undefined':
            return 'undefined';
        case 'Function':
            return 'function unknown(){}';
        default: {
            var partial_1 = [];
            var v_1;
            var keys = Object.keys(value);
            keys.forEach(function (k) {
                v_1 = stringify(value[k], __assign(__assign({}, options), { key: k }));
                if (v_1) {
                    partial_1.push(quoteKeys(k) + ':' + v_1);
                }
            }); // Join all the member texts together, separated with commas,
            // and wrap them in braces.
            if (partial_1.length === 0) {
                v_1 = '{}';
            }
            else {
                v_1 = '{' + partial_1.join(',') + '}';
            }
            return v_1;
        }
    }
} // If the JSON object does not yet have a stringify method, give it one.
exports.stringify = stringify;
exports.BJSON = new BJSONConstructor();
