"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.formatGraphQL = formatGraphQL;

var _graphql = require("graphql");

function formatGraphQL(schemaSdl) {
  var node = walkAST((0, _graphql.parse)(schemaSdl));
  return (0, _graphql.print)(node);
}

function sortSchema(key, value) {
  if (key === 'definitions' || key === 'fields' || key === 'arguments') {
    return value.slice().sort(function (a, b) {
      if (a.kind === 'SchemaDefinition') {
        return -1;
      }

      if (b.kind === 'SchemaDefinition') {
        return 1;
      }

      return a.name.value.localeCompare(b.name.value);
    });
  }

  return value;
}

function walkAST(node) {
  var key = node.kind === 'Document' ? 'definitions' : '';

  if (!key) {
    return node;
  }

  if (!Array.isArray(node[key])) {
    return node;
  }

  node[key] = sortSchema(key, node[key]).map(function (child) {
    return walkAST(child);
  });
  return node;
}
//# sourceMappingURL=formatGraphQL.js.map