"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.invariant = invariant;
exports.invariantType = invariantType;
exports.nonNullValues = nonNullValues;
exports.notNull = notNull;

var _RuntimeError = require("./RuntimeError");

var _ensureArray = require("./ensureArray");

var _getTypeName = require("./getTypeName");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

function nonNullValues(object) {
  var customMessage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  if (!object || _typeof(object) !== 'object') {
    throw new _RuntimeError.RuntimeError("".concat(object, " is not a valid object"), {
      input: object
    });
  }

  var errors = [];
  Object.entries(object).forEach(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        k = _ref2[0],
        v = _ref2[1];

    if (v === null || v === undefined) {
      errors.push("expected non nullable value for property \"".concat(k, "\", found ").concat(v, "."));
    }
  });

  if (errors.length) {
    if (customMessage) {
      errors[0] = customMessage;
    }

    throw new _RuntimeError.RuntimeError(errors.join('\n'), {
      input: object
    }, 2);
  }

  return object;
}

function notNull(input) {
  var appendErrorMessage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  if (input === null || input === undefined) {
    var message = "Expected non null value, but received ".concat(input, ".");

    if (appendErrorMessage) {
      message = "".concat(appendErrorMessage, " ").concat(message);
    }

    throw new _RuntimeError.RuntimeError(message, {
      input: input
    }, 2);
  }

  return input;
}

function invariant(truthy) {
  var errorMessage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var details = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  if (!truthy) {
    throw new _RuntimeError.RuntimeError(errorMessage, details, 2);
  }
}

function invariantType(object, type, extraInfo) {
  var depth = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 2;
  var skipLines = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 3;
  var typeArr = (0, _ensureArray.ensureArray)(type).map(function (el) {
    return el.toLowerCase();
  });
  Object.entries(object).forEach(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        key = _ref4[0],
        value = _ref4[1];

    var foundType = (0, _getTypeName.getTypeName)(value);
    var validType = typeArr.includes(foundType.toLowerCase());
    var invalidNull = value === null && !typeArr.includes('null');

    if (invalidNull || !validType) {
      throw new _RuntimeError.RuntimeError("Expected \"".concat(key, "\", to be of type \"").concat(type, "\", found ").concat(foundType, " ").concat(value), extraInfo, skipLines, depth);
    }
  });
  return true;
}
//# sourceMappingURL=invariant.js.map