"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NodeLogger = void 0;
exports.inspect = inspect;

var util = _interopRequireWildcard(require("util"));

var _logLevels = require("./logLevels");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var ALLOWED_LOG_LEVELS = (0, _logLevels.getLogLevelInfo)();
var logger = {
  error: console.error,
  debug: console.debug,
  info: console.info,
  warning: console.warn,
  crit: console.error
};

var NodeLogger = /*#__PURE__*/function () {
  // util on tests;
  function NodeLogger() {
    var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    _classCallCheck(this, NodeLogger);

    _defineProperty(this, "prefix", void 0);

    this.prefix = prefix;
  }

  _createClass(NodeLogger, [{
    key: "object",
    value: function object(_object) {
      var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;
      return this.log('info', inspect(_object, depth));
    }
  }, {
    key: "error",
    value: function error(err, extraInfo) {
      this.log('error', err, extraInfo);
    }
  }, {
    key: "criticalError",
    value: function criticalError(err, extraInfo) {
      return this.log('crit', err, extraInfo);
    }
  }, {
    key: "warn",
    value: function warn(data) {
      this.log('warning', data);
    }
  }, {
    key: "info",
    value: function info(data) {
      this.log('info', data);
    }
  }, {
    key: "debug",
    value: function debug(data) {
      this.log('debug', data);
    }
  }, {
    key: "log",
    value: function log(level, data, extraInfo) {
      NodeLogger.lastLogged = {
        data: data,
        extraInfo: extraInfo
      };

      if (!_logLevels.LogLevels.includes(level)) {
        console.trace("INVALID_LOG_LEVEL ".concat(level));
        return;
      }

      if (!ALLOWED_LOG_LEVELS.has(level)) return;
      var prefix = "".concat(this.prefix);
      var payload = [prefix];

      if (typeof data !== 'string') {
        data = inspect(data);
      }

      payload.push(data);

      try {
        logger[level](payload.join('\n'), extraInfo);
      } catch (e) {
        console.trace(e);
      }
    }
  }]);

  return NodeLogger;
}();

exports.NodeLogger = NodeLogger;

_defineProperty(NodeLogger, "lastLogged", null);

_defineProperty(NodeLogger, "logError", function (err, extraInfo) {
  var logger = new NodeLogger('');
  logger.error(err, extraInfo);
});

_defineProperty(NodeLogger, "logCriticalError", function (err, extraInfo) {
  var logger = new NodeLogger('');
  logger.criticalError(err, extraInfo);
});

_defineProperty(NodeLogger, "log", function (level, data) {
  var logger = new NodeLogger('');
  logger.log(level, data);
});

_defineProperty(NodeLogger, "logWarning", function (data) {
  var logger = new NodeLogger('');
  logger.warn(data);
});

_defineProperty(NodeLogger, "logInfo", function (data) {
  var logger = new NodeLogger('');
  logger.info(data);
});

_defineProperty(NodeLogger, "debug", function (data) {
  var logger = new NodeLogger('');
  logger.debug(data);
});

function inspect(arg) {
  var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;
  return util.inspect(arg, {
    depth: depth
  });
}
//# sourceMappingURL=nodeLogger.js.map