"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Contextify = void 0;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var vm_1 = __importDefault(require("vm"));
var useProcess_1 = require("../useProcess");
var builtin_modules_1 = require("./builtin-modules");
var ORIGINAL_REQUIRE = require;
var GET_PATHS = ORIGINAL_REQUIRE.resolve.paths;
var IS_PROD_ENV = ((_a = useProcess_1.Process.env) === null || _a === void 0 ? void 0 : _a.NODE_ENV) === 'production';
var Contextify = /** @class */ (function () {
    function Contextify(options) {
        var _this = this;
        this.allBranches = new Set();
        this.filePathCache = Object.create(null);
        this.contextModuleName = 'use-task-payload';
        this.getFileContents = function (filename) { return _this._readFileFromDisk(filename); };
        this.resolveModule = function (path, currentDirname) {
            return _this._findNodeModulePath(path, currentDirname);
        };
        this.prepareRequire = function (config) {
            var _a = _this, persistentCache = _a.persistentCache, mock = _a.mock;
            var currentDirname = config.currentDirname, _b = config.transitiveCache, transitiveCache = _b === void 0 ? Object.create(null) : _b, payload = config.payload;
            var self = _this;
            var vmContext = (function () {
                var vmContext = config.vmContext, sandbox = config.sandbox;
                if (vmContext)
                    return vmContext;
                if (!sandbox)
                    return null;
                if (sandbox === true)
                    return Contextify.prepareContext({});
                if (vm_1.default.isContext(sandbox))
                    return sandbox;
                return Contextify.prepareContext(sandbox);
            })();
            return function require(moduleName) {
                var filename = self.resolveModule(moduleName, currentDirname);
                var mocked = mock === null || mock === void 0 ? void 0 : mock({ moduleName: moduleName, filename: filename, payload: payload });
                if (mocked !== undefined) {
                    transitiveCache[filename] = mocked;
                    return mocked;
                }
                if (transitiveCache[filename]) {
                    return transitiveCache[filename].exports;
                }
                if (moduleName === self.contextModuleName) {
                    var module_1 = {
                        exports: function () { return payload; },
                    };
                    transitiveCache[filename] = module_1;
                    return module_1.exports;
                }
                if ((0, builtin_modules_1.isBuiltInModule)(moduleName)) {
                    return ORIGINAL_REQUIRE(moduleName);
                }
                if (!filename) {
                    throw new Error("Cannot find module ".concat(moduleName));
                }
                var closure = persistentCache[filename];
                if (!closure) {
                    var code = self._readFileWrapped(filename);
                    var script = new vm_1.default.Script(code, {
                        filename: filename,
                        displayErrors: false,
                    });
                    closure = vmContext
                        ? script.runInContext(vmContext, {
                            filename: filename,
                            displayErrors: true,
                        })
                        : script.runInThisContext({
                            filename: filename,
                            displayErrors: true,
                        });
                    if (!vmContext) {
                        persistentCache[filename] = closure;
                    }
                }
                var dirname = path_1.default.dirname(filename);
                var module = {
                    filename: filename,
                    exports: {},
                    require: self.prepareRequire({
                        currentDirname: dirname,
                        transitiveCache: transitiveCache,
                        payload: payload,
                        vmContext: vmContext,
                    }),
                };
                transitiveCache[filename] = module;
                var $TASK_MODULE_UTILS = {
                    sourceMaps: function () {
                        if (!self.allowSourcemaps)
                            return;
                        if (transitiveCache.installedSourcemaps)
                            return;
                        transitiveCache.installedSourcemaps = true;
                        function retrieveFile(filename) {
                            return (self.persistentCache["".concat(filename, ".wrapped_module_contents")] ||
                                null);
                        }
                        require('source-map-support').install({
                            retrieveFile: retrieveFile,
                            isTask: true,
                        });
                    },
                };
                closure(module.exports, module.require, module, filename, dirname, $TASK_MODULE_UTILS);
                return module.exports;
            };
        };
        this._readFileWrapped = function (filename) {
            var _a;
            var KEY = "".concat(filename, ".wrapped_module_contents");
            if (_this.persistentCache[KEY])
                return _this.persistentCache[KEY];
            var extension = path_1.default.extname(filename);
            var code;
            if ((_a = _this.buildByExtension) === null || _a === void 0 ? void 0 : _a[extension]) {
                code = _this.buildByExtension[extension](filename);
            }
            else {
                code = _this.getFileContents(filename);
            }
            return (_this.persistentCache[KEY] = wrapModuleCode(code));
        };
        this._findNodeModulePath = function (path, currentDirname) {
            if (path === _this.contextModuleName)
                return _this.contextModuleName;
            var cacheKey = "".concat(currentDirname, "_").concat(path);
            if (_this.filePathCache[cacheKey])
                return _this.filePathCache[cacheKey];
            var filePath = ORIGINAL_REQUIRE.resolve(path, {
                paths: __spreadArray([currentDirname], __read((GET_PATHS(currentDirname) || [])), false),
            });
            return (_this.filePathCache[cacheKey] = filePath);
        };
        this.diskFilesCache = Object.create(null);
        this._readFileFromDisk = function (filename) {
            var KEY = "".concat(filename, ".plain_disk_content");
            if (_this.diskFilesCache[KEY])
                return _this.diskFilesCache[KEY];
            return (_this.diskFilesCache[KEY] = fs_1.default.readFileSync(filename, 'utf8'));
        };
        this.persistentCache = options.persistentCache || Object.create(null);
        this.buildByExtension = options.buildByExtension;
        this.allowSourcemaps =
            typeof options.allowSourcemaps !== 'undefined'
                ? options.allowSourcemaps
                : !IS_PROD_ENV;
        this.mock = options.mock || null;
        if (options.contextModuleName) {
            this.contextModuleName = options.contextModuleName;
        }
        if (options.getFileContents) {
            this.getFileContents = options.getFileContents;
        }
        if (options.resolveModule) {
            var resolveModule_1 = options.resolveModule;
            var self_1 = this;
            this.resolveModule = function (path, currentDirname) {
                if ((0, builtin_modules_1.isBuiltInModule)(path))
                    return path;
                if (path === self_1.contextModuleName)
                    return self_1.contextModuleName;
                var result = resolveModule_1(path, currentDirname);
                if (!result) {
                    throw new Error("Cannot find module ".concat(path, " - currentDirname: ").concat(currentDirname));
                }
                return result;
            };
        }
    }
    Contextify.prepareContext = function (sandbox) {
        return vm_1.default.createContext(__assign({ global: { isContextifySandbox: true }, isTaskVMSandbox: true, process: process, console: console, version: parseInt(process.versions.node.split('.')[0]), setTimeout: setTimeout, setInterval: setInterval, setImmediate: setImmediate, clearTimeout: clearTimeout, clearInterval: clearInterval, clearImmediate: clearImmediate, String: String, Number: Number, Buffer: Buffer, Boolean: Boolean, Array: Array, Date: Date, Error: Error, EvalError: EvalError, RangeError: RangeError, ReferenceError: ReferenceError, SyntaxError: SyntaxError, TypeError: TypeError, URIError: URIError, RegExp: RegExp, Function: Function, Object: Object, Proxy: Proxy, Reflect: Reflect, Map: Map, WeakMap: WeakMap, Set: Set, WeakSet: WeakSet, Promise: Promise, Symbol: Symbol }, sandbox));
    };
    return Contextify;
}());
exports.Contextify = Contextify;
function wrapModuleCode(code, strict) {
    if (strict === void 0) { strict = true; }
    var MODULE_PREFIX = '(function (exports, require, module, __filename, __dirname, $TASK_MODULE_UTILS) { ';
    var STRICT_MODULE_PREFIX = MODULE_PREFIX + '"use strict";$TASK_MODULE_UTILS.sourceMaps();\n';
    var MODULE_SUFFIX = '\n});';
    var prefix = strict ? STRICT_MODULE_PREFIX : MODULE_PREFIX;
    return prefix + code + MODULE_SUFFIX;
}
