"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProxy = void 0;
function createProxy(thunk, options) {
    var data;
    var isResolved = false;
    var run = function () {
        if (isResolved)
            return data;
        isResolved = true;
        data = typeof thunk === 'function' ? thunk() : thunk;
        return data;
    };
    var proxy = new Proxy({}, {
        get: function (_o, k) {
            if (options === null || options === void 0 ? void 0 : options.onGet) {
                var res = options.onGet(k);
                if (res !== null)
                    return res;
            }
            var realValue = run();
            return realValue[k];
        },
        set: function (_o, k, v) {
            if (options === null || options === void 0 ? void 0 : options.onSet) {
                var res = options.onSet(k, v);
                if (res !== null)
                    return res;
            }
            run()[k] = v;
            return true;
        },
        has: function (_o, k) {
            if (options === null || options === void 0 ? void 0 : options.onHas) {
                var res = options.onHas(k);
                if (res !== null)
                    return res;
            }
            return k in run();
        },
        deleteProperty: function (_o, k) {
            delete run()[k];
            return true;
        },
        ownKeys: function () {
            return Reflect.ownKeys(run());
        },
        defineProperty: function (_o, k, d) {
            return Object.defineProperty(run(), k, d);
        },
        // @ts-expect-error
        getOwnPropertyNames: function () {
            return Object.getOwnPropertyNames(run());
        },
        getOwnPropertyDescriptor: function (_o, k) {
            return Object.getOwnPropertyDescriptor(run(), k);
        },
    });
    return proxy;
}
exports.createProxy = createProxy;
