"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.invariantType = exports.invariant = exports.notNull = exports.nonNullValues = void 0;
/**
 * Validates an object for keys with null or undefined values
 * @param object
 */
var RuntimeError_1 = require("./RuntimeError");
var ensureArray_1 = require("./ensureArray");
var getTypeName_1 = require("./getTypeName");
function nonNullValues(object, customMessage) {
    if (customMessage === void 0) { customMessage = ''; }
    if (!object || typeof object !== 'object') {
        throw new RuntimeError_1.RuntimeError("".concat(object, " is not a valid object"), {
            input: object,
        });
    }
    var errors = [];
    Object.entries(object).forEach(function (_a) {
        var _b = __read(_a, 2), k = _b[0], v = _b[1];
        if (v === null || v === undefined) {
            errors.push("expected non nullable value for property \"".concat(k, "\", found ").concat(v, "."));
        }
    });
    if (errors.length) {
        if (customMessage) {
            errors[0] = customMessage;
        }
        throw new RuntimeError_1.RuntimeError(errors.join('\n'), { input: object }, 2);
    }
    return object;
}
exports.nonNullValues = nonNullValues;
function notNull(input, appendErrorMessage) {
    if (appendErrorMessage === void 0) { appendErrorMessage = ''; }
    if (input === null || input === undefined) {
        var message = "Expected non null value, but received ".concat(input, ".");
        if (appendErrorMessage) {
            message = "".concat(appendErrorMessage, " ").concat(message);
        }
        throw new RuntimeError_1.RuntimeError(message, { input: input }, 2);
    }
    return input;
}
exports.notNull = notNull;
function invariant(truthy, errorMessage, details) {
    if (errorMessage === void 0) { errorMessage = ''; }
    if (details === void 0) { details = null; }
    if (!truthy) {
        throw new RuntimeError_1.RuntimeError(errorMessage, details, 2);
    }
}
exports.invariant = invariant;
function invariantType(object, type, extraInfo, depth, skipLines) {
    if (depth === void 0) { depth = 2; }
    if (skipLines === void 0) { skipLines = 3; }
    var typeArr = (0, ensureArray_1.ensureArray)(type).map(function (el) { return el.toLowerCase(); });
    Object.entries(object).forEach(function (_a) {
        var _b = __read(_a, 2), key = _b[0], value = _b[1];
        var foundType = (0, getTypeName_1.getTypeName)(value);
        var validType = typeArr.includes(foundType.toLowerCase());
        var invalidNull = value === null && !typeArr.includes('null');
        if (invalidNull || !validType) {
            throw new RuntimeError_1.RuntimeError("Expected \"".concat(key, "\", to be of type \"").concat(type, "\", found ").concat(foundType, " ").concat(value), extraInfo, skipLines, depth);
        }
    });
    return true;
}
exports.invariantType = invariantType;
