function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { dateSerialize } from './dateSerialize';
import { getTypeName } from './getTypeName';
export var SEP = 'ː';

function withSEP(txt) {
  return "".concat(SEP).concat(txt).concat(SEP);
}

export var BJSON_UNDEFINED = withSEP('undef');
export var BJSON_FUNCTION = withSEP('func');
export class Serializer {
  constructor(formatter) {
    _defineProperty(this, "name", void 0);

    _defineProperty(this, "sep", void 0);

    _defineProperty(this, "blockStart", void 0);

    _defineProperty(this, "blockEnd", void 0);

    _defineProperty(this, "paramSep", void 0);

    _defineProperty(this, "formatter", void 0);

    this.formatter = formatter;
    this.name = formatter.name;
    this.sep = Serializer.SEP;
    this.paramSep = this.sep;
    this.blockStart = "".concat(this.sep).concat(this.name).concat(this.sep, "(");
    this.blockEnd = ")";
  }

  stringify(value) {
    var tested = this.formatter.match(value);
    if (tested === undefined) return;

    var wrap = txt => "".concat(this.blockStart).concat(txt).concat(this.blockEnd);

    if (this.formatter.jsonReplacer) {
      return this.formatter.jsonReplacer;
    }

    var params = this.formatter.getParams(tested);
    return wrap(params.join(this.paramSep));
  }

  parse(value) {
    var _value$toString;

    var str = value === null || value === void 0 ? void 0 : (_value$toString = value.toString) === null || _value$toString === void 0 ? void 0 : _value$toString.call(value);
    if (typeof str !== 'string') return;

    if (this.formatter.jsonReplacer !== undefined && value === this.formatter.jsonReplacer) {
      return {
        value: this.formatter.hydrate()
      };
    }

    if (!(str.startsWith(this.blockStart) && str.endsWith(this.blockEnd))) {
      return;
    }

    var params = value.slice(this.blockStart.length).slice(0, -this.blockEnd.length).split(this.paramSep);
    return {
      value: this.formatter.hydrate(...params)
    };
  }

}

_defineProperty(Serializer, "SEP", SEP);

export class BJSONConstructor {
  constructor() {
    var _this = this;

    var _options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _defineProperty(this, "serializers", []);

    _defineProperty(this, "stringify", function (value) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var {
        handler,
        quoteValues,
        quoteKeys
      } = options;
      var self = _this;
      var str = stringify(value, _objectSpread(_objectSpread({}, options), {}, {
        quoteValues,
        quoteKeys,
        defaultHandler: _ref => {
          var {
            value,
            options
          } = _ref;

          var serializer = _this.serializers.find(el => el.formatter.match(value) !== undefined);

          var handled = handler === null || handler === void 0 ? void 0 : handler(_objectSpread(_objectSpread({}, options), {}, {
            value,
            self,
            serializer
          }));
          if (handled !== undefined) return handled;
          if (!serializer) return undefined;
          return serializer.stringify(value);
        }
      }));
      return "".concat(str);
    });

    _defineProperty(this, "parse", input => {
      return JSON.parse(input, (_key, value) => {
        for (var serializer of this.serializers) {
          var _match = serializer.parse(value);

          if (_match !== undefined) return _match.value;
        }

        return value;
      });
    });

    this.serializers = [...BJSONConstructor.serializers, ...(_options.extraSerializers || [])];
  }

  getSerializer(value) {
    var _value$split, _value$split$call;

    var name = value === null || value === void 0 ? void 0 : (_value$split = value.split) === null || _value$split === void 0 ? void 0 : (_value$split$call = _value$split.call(value, "".concat(Serializer.SEP))) === null || _value$split$call === void 0 ? void 0 : _value$split$call[0];
    if (!name) return;
    return this.serializers.find(s => s.name === name);
  }

  tsName(value) {
    var _this$getSerializer;

    return (_this$getSerializer = this.getSerializer(value)) === null || _this$getSerializer === void 0 ? void 0 : _this$getSerializer.formatter.tsName();
  }

}

_defineProperty(BJSONConstructor, "serializers", [new Serializer({
  name: 'Date',

  tsName() {
    return 'Date';
  },

  hydrate(value) {
    return new Date(value);
  },

  getParams(value) {
    return [value.toJSON()];
  },

  match(value) {
    return value instanceof Date ? value : typeof value === 'string' ? dateSerialize(value) || undefined : undefined;
  }

}), new Serializer({
  name: 'RegExp',

  tsName() {
    return 'RegExp';
  },

  hydrate(source, flags) {
    return new RegExp(source, flags);
  },

  getParams(value) {
    return [value.source, value.flags];
  },

  match(value) {
    return value instanceof RegExp ? value : undefined;
  }

}), new Serializer({
  name: 'NaN',

  tsName() {
    return 'NaN';
  },

  hydrate() {
    return NaN;
  },

  getParams() {
    return [];
  },

  match(value) {
    return typeof value === 'number' && isNaN(value) ? value : undefined;
  }

}), new Serializer({
  name: 'Undefined',

  tsName() {
    return 'undefined';
  },

  hydrate() {
    return undefined;
  },

  getParams() {
    return [];
  },

  match(value) {
    return value === undefined ? BJSON_UNDEFINED : undefined;
  },

  jsonReplacer: BJSON_UNDEFINED
}), new Serializer({
  name: 'Function',

  tsName() {
    return 'Function';
  },

  hydrate() {
    return function __unknown__() {};
  },

  getParams() {
    return [];
  },

  match(value) {
    return typeof value === 'function' ? value : undefined;
  },

  jsonReplacer: BJSON_FUNCTION
})]);

// some parts from meteor ejson
export function stringify(value) {
  var _options$key;

  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var {
    defaultHandler,
    quoteValues = v => JSON.stringify(v),
    quoteKeys = JSON.stringify
  } = options;
  var typeName = getTypeName(value);
  var info = {
    key: (_options$key = options === null || options === void 0 ? void 0 : options.key) !== null && _options$key !== void 0 ? _options$key : undefined
  };
  var handled = defaultHandler === null || defaultHandler === void 0 ? void 0 : defaultHandler({
    value,
    options
  });
  if (handled !== undefined) return quoteValues(handled, info);

  switch (typeName) {
    case 'String':
      return quoteValues(value, info);

    case 'NaN':
    case 'Infinity':
      return 'null';

    case 'Number':
      return quoteValues(value, info);

    case 'Boolean':
      return quoteValues(value, info);

    case 'Array':
      {
        var partial = [];
        var v; // Is the value an array?
        // The value is an array. Stringify every element. Use null as a
        // placeholder for non-JSON values.

        var length = value.length;

        for (var i = 0; i < length; i += 1) {
          partial[i] = stringify(value[i], _objectSpread(_objectSpread({}, options), {}, {
            key: i
          })) || 'null';
        } // Join all the elements together, separated with commas, and wrap
        // them in brackets.


        if (partial.length === 0) {
          v = '[]';
        } else {
          v = '[' + partial.join(',') + ']';
        }

        return v;
      }

    case 'Null':
      return 'null';

    case 'Undefined':
      return 'undefined';

    case 'Function':
      return 'function unknown(){}';

    default:
      {
        var _partial = [];

        var _v;

        var keys = Object.keys(value);
        keys.forEach(function (k) {
          _v = stringify(value[k], _objectSpread(_objectSpread({}, options), {}, {
            key: k
          }));

          if (_v) {
            _partial.push(quoteKeys(k) + ':' + _v);
          }
        }); // Join all the member texts together, separated with commas,
        // and wrap them in braces.

        if (_partial.length === 0) {
          _v = '{}';
        } else {
          _v = '{' + _partial.join(',') + '}';
        }

        return _v;
      }
  }
} // If the JSON object does not yet have a stringify method, give it one.

export var BJSON = new BJSONConstructor();
//# sourceMappingURL=BJSON.js.map