export function createProxy(thunk, options) {
  var data;
  var isResolved = false;

  var run = () => {
    if (isResolved) return data;
    isResolved = true;
    data = typeof thunk === 'function' ? thunk() : thunk;
    return data;
  };

  var proxy = new Proxy({}, {
    get(_o, k) {
      if (options !== null && options !== void 0 && options.onGet) {
        var res = options.onGet(k);
        if (res !== null) return res;
      }

      var realValue = run();
      return realValue[k];
    },

    set(_o, k, v) {
      if (options !== null && options !== void 0 && options.onSet) {
        var res = options.onSet(k, v);
        if (res !== null) return res;
      }

      run()[k] = v;
      return true;
    },

    has(_o, k) {
      if (options !== null && options !== void 0 && options.onHas) {
        var res = options.onHas(k);
        if (res !== null) return res;
      }

      return k in run();
    },

    deleteProperty(_o, k) {
      delete run()[k];
      return true;
    },

    ownKeys() {
      return Reflect.ownKeys(run());
    },

    defineProperty(_o, k, d) {
      return Object.defineProperty(run(), k, d);
    },

    // @ts-expect-error
    getOwnPropertyNames() {
      return Object.getOwnPropertyNames(run());
    },

    getOwnPropertyDescriptor(_o, k) {
      return Object.getOwnPropertyDescriptor(run(), k);
    }

  });
  return proxy;
}
//# sourceMappingURL=createProxy.js.map