/**
 * Validates an object for keys with null or undefined values
 * @param object
 */
import { RuntimeError } from './RuntimeError';
import { ensureArray } from './ensureArray';
import { getTypeName } from './getTypeName';
export function nonNullValues(object) {
  var customMessage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  if (!object || typeof object !== 'object') {
    throw new RuntimeError("".concat(object, " is not a valid object"), {
      input: object
    });
  }

  var errors = [];
  Object.entries(object).forEach(_ref => {
    var [k, v] = _ref;

    if (v === null || v === undefined) {
      errors.push("expected non nullable value for property \"".concat(k, "\", found ").concat(v, "."));
    }
  });

  if (errors.length) {
    if (customMessage) {
      errors[0] = customMessage;
    }

    throw new RuntimeError(errors.join('\n'), {
      input: object
    }, 2);
  }

  return object;
}
export function notNull(input) {
  var appendErrorMessage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  if (input === null || input === undefined) {
    var message = "Expected non null value, but received ".concat(input, ".");

    if (appendErrorMessage) {
      message = "".concat(appendErrorMessage, " ").concat(message);
    }

    throw new RuntimeError(message, {
      input
    }, 2);
  }

  return input;
}
export function invariant(truthy) {
  var errorMessage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var details = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  if (!truthy) {
    throw new RuntimeError(errorMessage, details, 2);
  }
}
export function invariantType(object, type, extraInfo) {
  var depth = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 2;
  var skipLines = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 3;
  var typeArr = ensureArray(type).map(el => el.toLowerCase());
  Object.entries(object).forEach(_ref2 => {
    var [key, value] = _ref2;
    var foundType = getTypeName(value);
    var validType = typeArr.includes(foundType.toLowerCase());
    var invalidNull = value === null && !typeArr.includes('null');

    if (invalidNull || !validType) {
      throw new RuntimeError("Expected \"".concat(key, "\", to be of type \"").concat(type, "\", found ").concat(foundType, " ").concat(value), extraInfo, skipLines, depth);
    }
  });
  return true;
}
//# sourceMappingURL=invariant.js.map