function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import * as util from 'util';
import { getLogLevelInfo, LogLevels } from './logLevels';
var ALLOWED_LOG_LEVELS = getLogLevelInfo();
var logger = {
  error: console.error,
  debug: console.debug,
  info: console.info,
  warning: console.warn,
  crit: console.error
};
export class NodeLogger {
  // util on tests;
  constructor() {
    var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    _defineProperty(this, "prefix", void 0);

    this.prefix = prefix;
  }

  object(object) {
    var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;
    return this.log('info', inspect(object, depth));
  }

  error(err, extraInfo) {
    this.log('error', err, extraInfo);
  }

  criticalError(err, extraInfo) {
    return this.log('crit', err, extraInfo);
  }

  warn(data) {
    this.log('warning', data);
  }

  info(data) {
    this.log('info', data);
  }

  debug(data) {
    this.log('debug', data);
  }

  log(level, data, extraInfo) {
    NodeLogger.lastLogged = {
      data,
      extraInfo
    };

    if (!LogLevels.includes(level)) {
      console.trace("INVALID_LOG_LEVEL ".concat(level));
      return;
    }

    if (!ALLOWED_LOG_LEVELS.has(level)) return;
    var prefix = "".concat(this.prefix);
    var payload = [prefix];

    if (typeof data !== 'string') {
      data = inspect(data);
    }

    payload.push(data);

    try {
      logger[level](payload.join('\n'), extraInfo);
    } catch (e) {
      console.trace(e);
    }
  }

}

_defineProperty(NodeLogger, "lastLogged", null);

_defineProperty(NodeLogger, "logError", (err, extraInfo) => {
  var logger = new NodeLogger('');
  logger.error(err, extraInfo);
});

_defineProperty(NodeLogger, "logCriticalError", (err, extraInfo) => {
  var logger = new NodeLogger('');
  logger.criticalError(err, extraInfo);
});

_defineProperty(NodeLogger, "log", (level, data) => {
  var logger = new NodeLogger('');
  logger.log(level, data);
});

_defineProperty(NodeLogger, "logWarning", data => {
  var logger = new NodeLogger('');
  logger.warn(data);
});

_defineProperty(NodeLogger, "logInfo", data => {
  var logger = new NodeLogger('');
  logger.info(data);
});

_defineProperty(NodeLogger, "debug", data => {
  var logger = new NodeLogger('');
  logger.debug(data);
});

export function inspect(arg) {
  var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;
  return util.inspect(arg, {
    depth
  });
}
//# sourceMappingURL=nodeLogger.js.map