export * from 'deep-object-diff';
export function objectDiffPaths(originObject, newObject) {
  var changes = [];

  function walkObject(_originObject, _newObject, paths) {
    var path = paths[paths.length - 1] || '';

    function _currentPath(parent, key) {
      var isArray = Array.isArray(parent);
      return isArray ? path + "[".concat(key, "]") : !path ? key : "".concat(path, ".").concat(key);
    }

    for (var key of Object.keys(_originObject)) {
      var currentPath = _currentPath(_originObject, key);

      if (!_newObject.hasOwnProperty(key)) {
        changes.push({
          kind: 'remove',
          newValue: undefined,
          oldValue: _originObject[key],
          path: currentPath,
          paths: [...paths, currentPath]
        });
      }
    }

    for (var [_key, value] of Object.entries(_newObject)) {
      if (value === _originObject[_key]) continue;

      var _currentPath2 = _currentPath(_newObject, _key);

      if (!_originObject.hasOwnProperty(_key)) {
        changes.push({
          kind: 'add',
          newValue: _newObject[_key],
          oldValue: undefined,
          path: _currentPath2,
          paths: [...paths, _currentPath2]
        });
      } else {
        if (value && typeof value === 'object' && _originObject[_key] && typeof _originObject[_key] === 'object') {
          walkObject(_originObject[_key], value, [...paths, _currentPath2]);
        } else {
          changes.push({
            kind: 'update',
            newValue: _newObject[_key],
            oldValue: _originObject[_key],
            path: _currentPath2,
            paths: [...paths, _currentPath2]
          });
        }
      }
    }
  }

  walkObject(originObject, newObject, []);
  return changes;
}
export var getObjectDiffPaths = objectDiffPaths;
//# sourceMappingURL=objectDiff.js.map