"use strict";
// copied from  ms@3.0.0-canary.1 to support node 14
Object.defineProperty(exports, "__esModule", { value: true });
exports.ms = void 0;
// Helpers.
var s = 1000;
var m = s * 60;
var h = m * 60;
var d = h * 24;
var w = d * 7;
var y = d * 365.25;
function msFn(value, options) {
    try {
        if (typeof value === 'string' && value.length > 0) {
            return parse(value);
        }
        else if (typeof value === 'number' && isFinite(value)) {
            return (options === null || options === void 0 ? void 0 : options.long) ? fmtLong(value) : fmtShort(value);
        }
        throw new Error('Value is not a string or number.');
    }
    catch (error) {
        var message = isError(error)
            ? "".concat(error.message, ". value=").concat(JSON.stringify(value))
            : 'An unknown error has occurred.';
        throw new Error(message);
    }
}
exports.ms = msFn;
/**
 * Parse the given string and return milliseconds.
 *
 * @param str - A string to parse to milliseconds
 * @returns The parsed value in milliseconds, or `NaN` if the string can't be
 * parsed
 */
function parse(str) {
    if (str.length > 100) {
        throw new Error('Value exceeds the maximum length of 100 characters.');
    }
    var match = /^(?<value>-?(?:\d+)?\.?\d+) *(?<type>milliseconds?|msecs?|ms|seconds?|secs?|s|minutes?|mins?|m|hours?|hrs?|h|days?|d|weeks?|w|years?|yrs?|y)?$/i.exec(str);
    // Named capture groups need to be manually typed today.
    // https://github.com/microsoft/TypeScript/issues/32098
    var groups = match === null || match === void 0 ? void 0 : match.groups;
    if (!groups) {
        return NaN;
    }
    var n = parseFloat(groups.value);
    var type = (groups.type || 'ms').toLowerCase();
    switch (type) {
        case 'years':
        case 'year':
        case 'yrs':
        case 'yr':
        case 'y':
            return n * y;
        case 'weeks':
        case 'week':
        case 'w':
            return n * w;
        case 'days':
        case 'day':
        case 'd':
            return n * d;
        case 'hours':
        case 'hour':
        case 'hrs':
        case 'hr':
        case 'h':
            return n * h;
        case 'minutes':
        case 'minute':
        case 'mins':
        case 'min':
        case 'm':
            return n * m;
        case 'seconds':
        case 'second':
        case 'secs':
        case 'sec':
        case 's':
            return n * s;
        case 'milliseconds':
        case 'millisecond':
        case 'msecs':
        case 'msec':
        case 'ms':
            return n;
        default:
            // This should never occur.
            throw new Error("The unit ".concat(type, " was matched, but no matching case exists."));
    }
}
exports.default = msFn;
/**
 * Short format for `ms`.
 */
function fmtShort(ms) {
    var msAbs = Math.abs(ms);
    if (msAbs >= d) {
        return "".concat(Math.round(ms / d), "d");
    }
    if (msAbs >= h) {
        return "".concat(Math.round(ms / h), "h");
    }
    if (msAbs >= m) {
        return "".concat(Math.round(ms / m), "m");
    }
    if (msAbs >= s) {
        return "".concat(Math.round(ms / s), "s");
    }
    return "".concat(ms, "ms");
}
/**
 * Long format for `ms`.
 */
function fmtLong(ms) {
    var msAbs = Math.abs(ms);
    if (msAbs >= d) {
        return plural(ms, msAbs, d, 'day');
    }
    if (msAbs >= h) {
        return plural(ms, msAbs, h, 'hour');
    }
    if (msAbs >= m) {
        return plural(ms, msAbs, m, 'minute');
    }
    if (msAbs >= s) {
        return plural(ms, msAbs, s, 'second');
    }
    return "".concat(ms, " ms");
}
/**
 * Pluralization helper.
 */
function plural(ms, msAbs, n, name) {
    var isPlural = msAbs >= n * 1.5;
    return "".concat(Math.round(ms / n), " ").concat(name).concat(isPlural ? 's' : '');
}
/**
 * A type guard for errors.
 *
 * @param value - The value to test
 * @returns A boolean `true` if the provided value is an Error-like object
 */
function isError(value) {
    return typeof value === 'object' && value !== null && 'message' in value;
}
