"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspect = exports.NodeLogger = void 0;
var util = __importStar(require("util"));
var logLevels_1 = require("./logLevels");
var ALLOWED_LOG_LEVELS = (0, logLevels_1.getLogLevelInfo)();
var logger = {
    error: console.error,
    debug: console.debug,
    info: console.info,
    warning: console.warn,
    crit: console.error,
};
var NodeLogger = /** @class */ (function () {
    function NodeLogger(prefix) {
        if (prefix === void 0) { prefix = ''; }
        this.prefix = prefix;
    }
    NodeLogger.prototype.object = function (object, depth) {
        if (depth === void 0) { depth = 5; }
        return this.log('info', inspect(object, depth));
    };
    NodeLogger.prototype.error = function (err, extraInfo) {
        this.log('error', err, extraInfo);
    };
    NodeLogger.prototype.criticalError = function (err, extraInfo) {
        return this.log('crit', err, extraInfo);
    };
    NodeLogger.prototype.warn = function (data) {
        this.log('warning', data);
    };
    NodeLogger.prototype.info = function (data) {
        this.log('info', data);
    };
    NodeLogger.prototype.debug = function (data) {
        this.log('debug', data);
    };
    NodeLogger.prototype.log = function (level, data, extraInfo) {
        NodeLogger.lastLogged = { data: data, extraInfo: extraInfo };
        if (!logLevels_1.LogLevels.includes(level)) {
            console.trace("INVALID_LOG_LEVEL ".concat(level));
            return;
        }
        if (!ALLOWED_LOG_LEVELS.has(level))
            return;
        var prefix = "".concat(this.prefix);
        var payload = [prefix];
        if (typeof data !== 'string') {
            data = inspect(data);
        }
        payload.push(data);
        try {
            logger[level](payload.join('\n'), extraInfo);
        }
        catch (e) {
            console.trace(e);
        }
    };
    NodeLogger.lastLogged = null; // util on tests;
    NodeLogger.logError = function (err, extraInfo) {
        var logger = new NodeLogger('');
        logger.error(err, extraInfo);
    };
    NodeLogger.logCriticalError = function (err, extraInfo) {
        var logger = new NodeLogger('');
        logger.criticalError(err, extraInfo);
    };
    NodeLogger.log = function (level, data) {
        var logger = new NodeLogger('');
        logger.log(level, data);
    };
    NodeLogger.logWarning = function (data) {
        var logger = new NodeLogger('');
        logger.warn(data);
    };
    NodeLogger.logInfo = function (data) {
        var logger = new NodeLogger('');
        logger.info(data);
    };
    NodeLogger.debug = function (data) {
        var logger = new NodeLogger('');
        logger.debug(data);
    };
    return NodeLogger;
}());
exports.NodeLogger = NodeLogger;
function inspect(arg, depth) {
    if (depth === void 0) { depth = 5; }
    return util.inspect(arg, { depth: depth });
}
exports.inspect = inspect;
