"use strict";
// https://github.com/facebook/react/blob/3e94bce765d355d74f6a60feb4addb6d196e3482/packages/react-dom/src/server/ReactThreadIDAllocator.js#L13-L12
// Allocates a new index for each request. Tries to stay as compact as possible so that these
// indexes can be used to reference a tightly packed array. As opposed to being used in a Map.
// The first allocated index is 1.
Object.defineProperty(exports, "__esModule", { value: true });
exports.freeThreadID = exports.allocThreadID = void 0;
var invariant_1 = require("./invariant");
var nextAvailableThreadIDs = new Uint16Array(16);
for (var i = 0; i < 15; i++) {
    nextAvailableThreadIDs[i] = i + 1;
}
nextAvailableThreadIDs[15] = 0;
function growThreadCountAndReturnNextAvailable() {
    var oldArray = nextAvailableThreadIDs;
    var oldSize = oldArray.length;
    var newSize = oldSize * 2;
    (0, invariant_1.invariant)(newSize <= 0x10000, 'Maximum number of concurrent threads exceeded. ');
    var newArray = new Uint16Array(newSize);
    newArray.set(oldArray);
    nextAvailableThreadIDs = newArray;
    nextAvailableThreadIDs[0] = oldSize + 1;
    for (var i = oldSize; i < newSize - 1; i++) {
        nextAvailableThreadIDs[i] = i + 1;
    }
    nextAvailableThreadIDs[newSize - 1] = 0;
    return oldSize;
}
function allocThreadID() {
    var nextID = nextAvailableThreadIDs[0];
    if (nextID === 0) {
        return growThreadCountAndReturnNextAvailable();
    }
    nextAvailableThreadIDs[0] = nextAvailableThreadIDs[nextID];
    return nextID;
}
exports.allocThreadID = allocThreadID;
function freeThreadID(id) {
    nextAvailableThreadIDs[id] = nextAvailableThreadIDs[0];
    nextAvailableThreadIDs[0] = id;
}
exports.freeThreadID = freeThreadID;
